library ieee;
  use ieee.std_logic_1164.all;
  use ieee.numeric_std.all;
  
entity ReadRam is
  port 
  (
		CLK 					: in  	std_logic;
		RST 					: in  	std_logic;

		--Avalon       
		chipselect				: in 		std_logic;
		read 					: in 		std_logic;
		address 				: in 		std_logic_vector(13 downto 0);		--outram 14 bits
		readdata 				: out 	std_logic_vector(31 downto 0);	
		
		--OutputRam
		raddr					: out 	std_logic_vector(13 downto 0);			--outram 14 bits
		q						: in 		std_logic_vector(7 downto 0)	
	);
	
end ReadRam;

architecture rtl of ReadRam is

	signal memdata : std_logic_vector(7 downto 0);
	signal ready   : integer := 0;

begin

	process(CLK)
	begin
		if rising_edge(CLK) then
				
			if RST = '1' then
				raddr 			<= (others => '0');
				readdata 		<= (others => '0');
				ready 			<= 0;				
			end if;
			
			if chipselect = '1' and read = '1' then
				ready 			<= ready + 1;
				raddr 			<= address;
				
				if ready > 1 then
					readdata 	<= X"000000" & q;
				else
					readdata 	<= (others => '0');
				end if;
				
			else
				raddr 			<= (others => '0');
				readdata 		<= (others => '0');
				ready 			<= 0;
			end if;
			
		end if;
	end process;

end rtl;
