library ieee;
  use ieee.std_logic_1164.all;
  
entity SavePixel is
  port 
  (

		CLK 				: in  	std_logic;
		RST 				: in  	std_logic;

		--Avalon       
		chipselect			: in 	std_logic;
		write 				: in 	std_logic;
		read 				: in 	std_logic;
		address 			: in 	std_logic_vector(7 downto 0);
		writedata 			: in 	std_logic_vector(31 downto 0);
		readdata 			: out 	std_logic_vector(31 downto 0);	
			
		-- HOST DATA
		iram_wdata          : out 	std_logic_vector(23 downto 0);	--C_PIXEL_BITS
		iram_wren           : out 	std_logic;
		fifo_almost_full    : in  	std_logic
		
	);
	
end SavePixel;

architecture rtl of SavePixel is

	signal asserted			: std_logic := '0';

begin

	process(CLK)
	begin
		if rising_edge(CLK) then			
			if RST = '1' then
				iram_wdata 	<= (others => '0');
				iram_wren 	<= '0';
				readdata	<= (others => '0');
				asserted 	<= '0';
			else
				if chipselect = '1' then
					if read = '1' then
						readdata 	<= X"0000000" & "000" & fifo_almost_full;
						iram_wren	<= '0';
						iram_wdata 	<= (others => '0');
						asserted 	<= '0';
					end if;
						
					if write = '1' then
						--must be asserted for 1 cycle
						if asserted = '0' then
							iram_wdata	<= writedata(23 downto 0);
							iram_wren	<= '1';					
							readdata	<= (others => '0');
							asserted 	<= '1';
						else
							iram_wdata	<= (others => '0');
							iram_wren	<= '0';					
							readdata	<= (others => '0');
						end if;
					end if;
				else
					iram_wdata 	<= (others => '0');
					iram_wren 	<= '0';
					readdata	<= (others => '0');
					asserted 	<= '0';
				end if;
			end if;
		end if;
	end process;

end rtl;
