#include <io.h>
#include <system.h>
#include <stdio.h>
#include <stdlib.h>
#include "gui.h"
#include "network.h"
#include "misc.h"
#include "jpeg_encoder.h"

#define SAVEPIXEL(data)            IOWR_32DIRECT(SAVEPIXEL_BASE, 0, (data))
#define READQUEUE                  IORD_32DIRECT(SAVEPIXEL_BASE, 0)
#define READBYTE(offset)           IORD_32DIRECT(READRAM_BASE, (offset) * 4)

#define SAVEREG(address, data)     IOWR_32DIRECT(AVALONTRANS_BASE, (address) *4, (data))
#define READREG(address)           IORD_32DIRECT(AVALONTRANS_BASE, (address) *4)

#define ENC_START_REG       0x0000
#define IMAGE_SIZE_REG      0x0004
#define ENC_STS_REG         0x000C
#define ENC_LENGTH_REG      0x0014
#define QUANTIZER_RAM_LUM   0x0100
#define QUANTIZER_RAM_CHR   0x0200

unsigned char jpeg_image[15000];
unsigned int jpeg_image_size;

const char luminance[] = {
   0x10, 0x0B, 0x0C, 0x0E, 0x0C, 0x0A, 0x10, 0x0E, 
   0x0D, 0x0E, 0x12, 0x11, 0x10, 0x13, 0x18, 0x28,
   0x1A, 0x18, 0x16, 0x16, 0x18, 0x31, 0x23, 0x25, 
   0x1D, 0x28, 0x3A, 0x33, 0x3D, 0x3C, 0x39, 0x33,
   0x38, 0x37, 0x40, 0x48, 0x5C, 0x4E, 0x40, 0x44, 
   0x57, 0x45, 0x37, 0x38, 0x50, 0x6D, 0x51, 0x57,
   0x5F, 0x62, 0x67, 0x68, 0x67, 0x3E, 0x4D, 0x71, 
   0x79, 0x70, 0x64, 0x78, 0x5C, 0x65, 0x67, 0x63   
};

const char chrominance[] = {
  0x11, 0x12, 0x12, 0x18, 0x15, 0x18, 0x2F, 0x1A, 
  0x1A, 0x2F, 0x63, 0x42, 0x38, 0x42, 0x63, 0x63,
  0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 
  0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
  0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 
  0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
  0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 
  0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63  
};

//const unsigned int image[];
//unsigned char raster[3][336][232];

//unsigned char raster[3][3][2];
//unsigned char jpeg_image[1];


int compress_jpeg(int num) {
    volatile unsigned int i, j, k, m, temp4, temp1, temp2, temp3;
      
    i = READREG(ENC_STS_REG); 
    printf("enc_status_reg = %x\n", i ); 
    
    i = READREG(ENC_START_REG);
    printf("enc_start_reg = %x\n", i );

    i = READREG(IMAGE_SIZE_REG);
    printf("image_size_reg = %x\n", i );
    
    printf("Writing Image size 336x232 pixels\n");
    SAVEREG(IMAGE_SIZE_REG,0x015000E8);
    
    i = READREG(IMAGE_SIZE_REG);
    printf("image_size_reg = %x\n", i );
    
    i = READREG(ENC_LENGTH_REG);
    printf("\nlength = %u\n", i);

    
    printf(" loading luminance RAM\n");
    for (j=0; j<64; j++) {
        SAVEREG(QUANTIZER_RAM_LUM+j*4, luminance[j]);
        printf(".");          
    }
    printf("\n luminance RAM saved\n");
    
    i = READREG(ENC_STS_REG); 
    printf("enc_status_reg = %x\n", i ); 
    
    printf(" Loading chrominance RAM\n");

    for (j=0; j<64; j++) {
        SAVEREG(QUANTIZER_RAM_CHR+j*4, chrominance[j]);
        printf(".");          
    }
    printf("\n chrominance RAM saved\n");
    
    i = READREG(ENC_STS_REG); 
    printf("enc_status_reg = %x\n", i ); 
    
    
    printf ("Starting encoder\n");
    SAVEREG(ENC_START_REG, 0x7);
    
    i = READREG(ENC_START_REG);
    printf("enc_start_reg = %x\n", i );
    
    i = READREG(ENC_STS_REG); 
    printf("enc_status_reg = %x\n", i ); 
    
    i = READREG(IMAGE_SIZE_REG);
    printf("image_size_reg = %x\n", i );
    
    i = READREG(ENC_STS_REG); 
    printf("enc_status_reg = %x\n", i ); 

    for (i=0; i<232; i++) { //y
        for (j=0; j<336; j++) { //x
            temp1 = saved_images[num][2][i][j] & 0xff;
            temp2 = (((unsigned int)saved_images[num][1][i][j])<<8) & 0xff00;
            temp3 = (((unsigned int)saved_images[num][0][i][j])<<16) & 0xff0000;
            temp4 = (temp3 >> 16) | temp2 | (temp1 << 16);
            //temp4 = ((unsigned int)saved_images[num][0][i][j])<<16 | saved_images[num][1][i][j] | saved_images[num][2][i][j];
            //printf("%x\n",temp4);
            SAVEPIXEL(temp4);
            
            if ((k = READQUEUE)) {
                m =10000;
                while (m--);  
            }
            //printf("%d, %d\n",i,j);
        }
        
        //getchar();
        m =1000;
        while (m--); 
        //printf(".");
    }

    printf("\n");
    k = 0;
    //getchar();
    //while (k < 2) {
        k = READREG(ENC_STS_REG);
        printf(".");     
        m =100000;
        while (m--);   
    //}
    
    i = READREG(ENC_STS_REG); 
    printf("\nenc_status_reg = %x\n", i ); 
            m =100000;
        while (m--); 
    
    i = READREG(ENC_LENGTH_REG);
    jpeg_image_size = i+2;
    printf("\nlength = %u\n", i);

    m=0;
    //printf("\n %05x:   ",m);
    for (j=0; j<jpeg_image_size; j++) {
        m++;
        k = READBYTE(j);
        jpeg_image[j]=k;
        //printf("%02X ",k);jpeg_image_size
        //printf("%02X ",jpeg_image[j]);
        if (m%16==0) printf("\n");   //printf("\n %05x:   ",m);
    }      
    
    i = READREG(ENC_STS_REG); 
    printf("\nenc_status_reg = %x\n", i ); 
    
    i = READREG(ENC_LENGTH_REG);
    printf("\nlength = %u\n", i);
        
    printf("\ndone\n");
    
    /* alex's code */
    // send a zero then filesize packet
    int zero = 0;
    printf("sending zero packet\n");
    network_send_packet((char*)&zero, sizeof(zero));
    gotosleep(1);    
    printf("sending filesize packet\n");
    network_send_packet((char*)&jpeg_image_size, sizeof(jpeg_image_size));
    gotosleep(1);
    printf("sending data\n");
    network_send_packet(jpeg_image, jpeg_image_size);    
    /* end alex's code */  
  return 0;
}


