/* TEAM-SYNTHESIS  */
#include <string.h>
#include <ctype.h>
#include "basic_io.h"
#include "DM9000A.h"
#include <alt_types.h>
#include "alt_up_ps2_port.h"
#include "ps2_keyboard.h"
#include "ps2_mouse.h"
#include "LCD.h"
#include "VGA.h"
#include "gui.h"
#include "jpeg_encoder.h"
#include "misc.h"
#include "network.h"

#define IMG_X_SIZE  550
#define IMG_Y_SIZE  360

#define MAX_MSG_LENGTH 800  // must be odd
#define DIV_LINE VGA_HEIGHT - 70
#define NUM_CHAT_LINES 24
#define USER_ROW 26

// Ethernet MAC address.  Choose the last three bytes yourself
unsigned char mac_address[6] = { 0x03, 0x62, 0x6E, 0x11, 0x02, 0x0F  };

unsigned int interrupt_number;

unsigned int receive_buffer_length;
unsigned char receive_buffer[1600];
unsigned char user_name[MAX_MSG_LENGTH];
unsigned char chat_buffer[NUM_CHAT_LINES][MAX_MSG_LENGTH];

KB_CODE_TYPE decode_mode;

#define UDP_PACKET_PAYLOAD_OFFSET 42
#define UDP_PACKET_LENGTH_OFFSET 38

#define UDP_PACKET_PAYLOAD (transmit_buffer + UDP_PACKET_PAYLOAD_OFFSET)

unsigned char transmit_buffer[MAX_MSG_LENGTH];
unsigned char transmit_buffer_template[] = {
  // Ethernet MAC header
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, // Destination MAC address
  0x01, 0x60, 0x6E, 0x11, 0x02, 0x0F, // Source MAC address
  0x08, 0x00,                        // Packet Type: 0x800 = IP
                          
  // IP Header
  0x45,                // version (IPv4), header length = 20 bytes
  0x00,                // differentiated services field
  0x00,0x9C,           // total length: 20 bytes for IP header +
                       // 8 bytes for UDP header + 128 bytes for payload
  0x3d, 0x35,          // packet ID
  0x00,                // flags
  0x00,                // fragment offset
  0x80,                // time-to-live
  0x11,                // protocol: 11 = UDP
  0xa3,0x43,           // header checksum: incorrect
  0xc0,0xa8,0x01,0x01, // source IP address
  0xc0,0xa8,0x01,0xff, // destination IP address
                          
  // UDP Header
  0x67,0xd9, // source port port (26585: garbage)
  0x27,0x2b, // destination port (10027: garbage)
  0x00,0x88, // length (136: 8 for UDP header + 128 for data)
  0x00,0x00, // checksum: 0 = none
                          
  // UDP payload
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67,
  0x74, 0x65, 0x73, 0x74, 0x20, 0x6d, 0x73, 0x67
};

//int send_value;
//int *sending;
int x, y;
int x_pos, y_pos;

int remote_img_num;
int remote_img_num2;

void fill_buffer() {
    UDP_PACKET_PAYLOAD[0] = 3;
    UDP_PACKET_PAYLOAD[1] = x & 0xff;
    UDP_PACKET_PAYLOAD[2] = x >> 8;
    UDP_PACKET_PAYLOAD[3] = y & 0xff;
    UDP_PACKET_PAYLOAD[4] = y >> 8;
    int p = 0;
    while (p < (MAX_MSG_LENGTH-7) && x < IMG_X_SIZE && y < IMG_Y_SIZE)
    {
        
        unsigned char byte_r = saved_images[remote_img_num][0][x][y];
        unsigned char byte_g = saved_images[remote_img_num][1][x][y];
        unsigned char byte_b = saved_images[remote_img_num][2][x][y];
        
        
        UDP_PACKET_PAYLOAD[p+7] = (((byte_g >> 2) & 0x7) << 5) | (byte_b >> 3);
        p++;
        UDP_PACKET_PAYLOAD[p+7] = ((byte_r >> 3) << 3) | (byte_g >> 5); 
        p++;
        x++;
        if (x == IMG_X_SIZE)
        {
            x = 0;
            y++;
            //printf("y: %d, %d, %d, %d\n", y, byte_r, byte_g, byte_b);
            //printf("%d, %d\n", UDP_PACKET_PAYLOAD[p+6], UDP_PACKET_PAYLOAD[p+5]);
        }
    }

    if (y == IMG_Y_SIZE)
    {
        printf("Sent\n");
        y = -1;
    }
    
    //printf("Sent (%d)\n", p);
    UDP_PACKET_PAYLOAD[5] = p & 0xff;
    UDP_PACKET_PAYLOAD[6] = p >> 8;
    TransmitPacket(transmit_buffer, UDP_PACKET_PAYLOAD_OFFSET + (p+6) + 1);
    
}

static void ethernet_interrupt_handler() {
  unsigned int receive_status;
  
  receive_status = ReceivePacket(receive_buffer, &receive_buffer_length);

  if (receive_status == DMFE_SUCCESS) {

#if 1
    
    /*printf("\n\nReceive Packet Length = %d", receive_buffer_length);
    for(i=0;i<receive_buffer_length;i++) {
      if (i%8==0) printf("\n");
      printf("0x%.2X,", receive_buffer[i]);
    }
    printf("\n");*/
#endif

    if (receive_buffer_length >= 14) {
      //  A real Ethernet packet
      if (receive_buffer[12] == 8 && receive_buffer[13] == 0 &&
      receive_buffer_length >= 34) {
    // An IP packet
    if (receive_buffer[23] == 0x11)
    {
      // A UDP packet
      if (receive_buffer_length >= UDP_PACKET_PAYLOAD_OFFSET)
      {
//----------------------------------------------------------------------------        
        //int i = atoi(receive_buffer + UDP_PACKET_PAYLOAD_OFFSET);
        int request = (receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[0];
        
        // type 1 = begin *sending 
        if (request == 1)// && *sending != 1)
        {
            remote_img_num2 = save_remote_image();
            x_pos = 0;
            y_pos = 0;
            printf("Receiving\n");
            //*sending = 1;
            // type 2 = ack 
            UDP_PACKET_PAYLOAD[0] = 2;
            TransmitPacket(transmit_buffer, UDP_PACKET_PAYLOAD_OFFSET + 1 + 1);
        }
        // type 3 = *sending pixel data
        else if (request == 3)// && *sending == 1)
        {
            x_pos = (receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[1];
            x_pos |=  (receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[2] << 8;
            y_pos = (receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[3];
            y_pos |=  (receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[4] << 8;
            int num = (receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[5];
            num |=  (receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[6] << 8;
            //printf("num (%d): ", num);
            int i = 7;
            while (i < num+7 && i < MAX_MSG_LENGTH && y_pos < IMG_Y_SIZE)
            {
                unsigned char byte_r, byte_g, byte_b;
                byte_b = (receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[i] & 0x1f << 3;
                byte_g = (((receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[i+1] & 0x7) << 5)
                    | (((receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[i] >> 5) << 2);
                byte_r = ((receive_buffer + UDP_PACKET_PAYLOAD_OFFSET)[i+1] >> 3) << 3;
                
                
                saved_images[remote_img_num2][0][x_pos][y_pos] = byte_r;
                saved_images[remote_img_num2][1][x_pos][y_pos] = byte_g;
                saved_images[remote_img_num2][2][x_pos][y_pos] = byte_b;
                
                i += 2;
                x_pos++;
                //printf("\nx_pos = %d\n", x);
                if (x_pos == IMG_X_SIZE)
                {
                    x_pos = 0;
                    //printf("y_pos = %d, %d, %d, %d\n", y_pos, byte_r, byte_g, byte_b);
                    y_pos++;
                }   
            }
            if (y == IMG_Y_SIZE)
            {
                printf("setting it to 0\n");
               // *sending = 0;
            }
            else
            {   
                // type 2 = received
                UDP_PACKET_PAYLOAD[0] = 2;
                TransmitPacket(transmit_buffer, UDP_PACKET_PAYLOAD_OFFSET + 1 + 1);
            }
            /*while (y != -1 && timer < 500000)
            {
                timer++;
            }
            printf("Done\n");
            sending = 0;*/
        }
        else if (request == 2 && y != -1) //&& *sending == 1 && y != -1)
        {
            //timer = 0;
            fill_buffer();
        }
        // type 3 = finished *sending
        else if (request == 4)// && *sending == 1)
        {
            printf("Finish receiving\n");
           // *sending = 0;
        }





//-----------------------------------------------------------          
      }
    } else {
      printf("Received non-UDP packet\n");
    }
      } else {
    printf("Received non-IP packet\n");
      }
    } else {
      printf("Malformed Ethernet packet\n");
    }

  } else {
    printf("Error receiving packet\n");
  }

  /* Display the number of interrupts on the LEDs */
  interrupt_number++;
  outport(SEG7_DISPLAY_BASE, interrupt_number);

  /* Clear the DM9000A ISR: PRS, PTS, ROS, ROOS 4 bits, by RW/C1 */
  dm9000a_iow(ISR, 0x3F);
              
  /* Re-enable DM9000A interrupts */
  dm9000a_iow(IMR, INTR_set);
}


void compute_checksum()
{
    // increment ip id
    int packet_id = (transmit_buffer[18] << 8) + transmit_buffer[19] + 1;
    transmit_buffer[18] = packet_id & 0xff00 >> 8;
    transmit_buffer[19] = packet_id & 0xff;
    printf("packed_id: %x\n", packet_id);
    
    int sum = 0;
    int i;
    for (i = 0; i < 10; i++)
    {
        if (i != 5)
            sum += (transmit_buffer[14 + i*2] << 8) + transmit_buffer[15 + i*2];
    }
    // turn to one's complement
    sum = (sum & 0xffff) + (sum >> 16);
    
    // complement
    sum = ~sum;
    
    transmit_buffer[24] = sum >> 8;
    transmit_buffer[25] = sum & 0x00ff;
    printf("checksum: %x\n", sum);
}

void handel_input()
{
  unsigned int curMsgChar = 0;
  alt_u8 key = 0;
  int status = 0;
  unsigned int packet_length;
  int capital = 0;
  UDP_PACKET_PAYLOAD[0] = 0;
  for (;;) {
    // wait for the user's input and get the make code
    status = read_make_code(&decode_mode, &key);
    if (key == 0xf0)
        printf("key up\n");
    
    if (status == PS2_SUCCESS) {
    
      // print out the result
      switch ( decode_mode ){
      case KB_ASCII_MAKE_CODE :
        key = (capital) ? key : tolower(key);
        printf("%c", key);
        
    if (curMsgChar < MAX_MSG_LENGTH) { 
      int i;
      int len = strlen(UDP_PACKET_PAYLOAD);
      UDP_PACKET_PAYLOAD[len+1] = 0;
      for (i = len; len > 0 && i > curMsgChar; i--)
      { 
        UDP_PACKET_PAYLOAD[i] = UDP_PACKET_PAYLOAD[i-1];
      }
      UDP_PACKET_PAYLOAD[curMsgChar] = key;
      curMsgChar++;
    }
    break ;
      case KB_LONG_BINARY_MAKE_CODE :
    // fall through
      case KB_BINARY_MAKE_CODE :
    switch (key) {
    case  0x5a: //enter key: send the msg 
      packet_length = 8 + 1;//curMsgChar;
      transmit_buffer[UDP_PACKET_LENGTH_OFFSET] = packet_length >> 8;
      transmit_buffer[UDP_PACKET_LENGTH_OFFSET + 1] = packet_length & 0xff;
      //compute_checksum();
      
      /*alt_u8 x, y;
      for (y = 1; y < 5; y++)
      {
        for (x = 1; x < 4; x++)
        {
            UDP_PACKET_PAYLOAD[0] = x;
            printf("%d\n", x);
            TransmitPacket(transmit_buffer, UDP_PACKET_PAYLOAD_OFFSET + 1 + 1);
        }
      }*/

      //int i;
      //for (i = 0; i < 1000; i++)
      //{
        if (capital == 2 && get_num_images() > 0)
        {
            remote_img_num = get_current_img_num();
            printf("Sending\n");
            y = 0;
            x = 0;
            UDP_PACKET_PAYLOAD[0] = 1;
            TransmitPacket(transmit_buffer, UDP_PACKET_PAYLOAD_OFFSET + 1 + 1);
        }
        if (capital == 1 && get_num_images() > 0)
        {
            
           printf("Compressing Image\n");
           
           compress_jpeg(get_current_img_num());

           
        }
        else if (get_num_images() < MAX_NUM_IMAGES)
        {
            save_image();
        }

      return; 
    case 0x29: //space key
      printf("key: space\n");
      freeze_toggle();
      break;
    default:
      // delete char if encounter delete key
      if (key == 0x66)
      {
        printf("key: delete\n");
        if (get_num_images() > 0)
            delete_current_image();
      }
    }
        // shift encountered
        if (key == 0x12)
        {
          printf("key: left shift\n");
          capital = 1;
        }
        if (key == 0x59)
        {
          printf("key: right shift\n");
          capital = 2;
        }
      if (key == 0x74)
      {
        printf("key: right\n");
        if (get_num_images() >= 1)
            send_image(2);
      }
      if (key == 0x6B)
      {
        printf("key: left\n");
        if (get_num_images() >= 1)
            send_image(1);
      }
      if (key == 0x75)
      {
        printf("key: up\n");
        full_screen(0);
      }
      if (key == 0x72)
      {
        printf("key: down\n");
        full_screen(1);
      }
        
    break ;
      case KB_BREAK_CODE :
        // shift encountered
        if (key == 0x59 || key == 0x12)
        {
          capital = 0;
        }
    //printf("break: %c\n", key);
      default :
    break ;
      }
    }
  }
}


int main(int argc, char* argv[])
{
  /* Added by alex */
  alex_init();

  network_set_source_ip(10, 0, 0, 15);
  network_set_dest_ip(10, 0, 0, 1);
  network_set_source_port(9931);
  network_set_dest_port(9930);
  /* end alex */
  
  //send_value = 0;
  //sending = &send_value;
  int i;
  for (i = 0; i < 128; i++)
  {
    transmit_buffer[i] = transmit_buffer_template[i];
  }
    
  int curMsgChar = 0;
  

  // Initalize the DM9000 and the Ethernet interrupt handler
  //DM9000_init(mac_address);
  //interrupt_number = 0;
  alt_irq_register(DM9000A_IRQ, NULL, (void*)ethernet_interrupt_handler); 
 
  printf("Ready to send messages\n");


  // Clear the payload
  for (curMsgChar=MAX_MSG_LENGTH-1; curMsgChar>0; curMsgChar--) {
    UDP_PACKET_PAYLOAD[curMsgChar] = 0;
  }


  for (;;)
  {
    handel_input();
  }

  printf("Program terminated normally\n");
  return 0;
        
 ErrorExit:
  printf("Program terminated with an error condition\n");

  return 1;
}
