#ifndef NETWORK_H_
#define NETWORK_H_

struct SPacket {
  char* data;
  unsigned int size;
};

typedef struct SPacket Packet;

extern const unsigned char SOURCE_MAC_ADDR[];
extern const unsigned char DEST_MAC_ADDR[];
extern const int ETHERNET_HEADER_SIZE;
extern const int IP_HEADER_SIZE;
extern const int UDP_HEADER_SIZE;

//#define MIN_PACKET_SIZE 64
#define MIN_PACKET_SIZE 128
#define NETWORK_MAX_PACKET_SIZE 1024

#define HEADER_OFFSET_SOURCE_MAC 0
#define HEADER_OFFSET_DEST_MAC 6

#define HEADER_OFFSET_IP_TOTAL_LENGTH_H 16
#define HEADER_OFFSET_IP_TOTAL_LENGTH_L 17

#define HEADER_OFFSET_PACKET_ID_H 18
#define HEADER_OFFSET_PACKET_ID_L 19

#define HEADER_OFFSET_PROTOCOL 23

#define HEADER_OFFSET_IP_CHECKSUM_H 24
#define HEADER_OFFSET_IP_CHECKSUM_L 25

// MSB
#define HEADER_OFFSET_SOURCE_IP_3 26
#define HEADER_OFFSET_SOURCE_IP_2 27
#define HEADER_OFFSET_SOURCE_IP_1 28
#define HEADER_OFFSET_SOURCE_IP_0 29

// MSB
#define HEADER_OFFSET_DEST_IP_3 30
#define HEADER_OFFSET_DEST_IP_2 31
#define HEADER_OFFSET_DEST_IP_1 32
#define HEADER_OFFSET_DEST_IP_0 33

#define HEADER_OFFSET_UDP 34
#define HEADER_OFFSET_SOURCE_PORT_H 34
#define HEADER_OFFSET_SOURCE_PORT_L 35

#define HEADER_OFFSET_DEST_PORT_H 36
#define HEADER_OFFSET_DEST_PORT_L 37

#define HEADER_OFFSET_UDP_LENGTH_H 38
#define HEADER_OFFSET_UDP_LENGTH_L 39

#define HEADER_OFFSET_UDP_CHECKSUM_H 40
#define HEADER_OFFSET_UDP_CHECKSUM_L 41

#define HEADER_OFFSET_UDP_DATA 42

#define NETWORK_UDP_PACKET_SIZE (ETHERNET_HEADER_SIZE + IP_HEADER_SIZE + UDP_HEADER_SIZE) 

void network_init();
void network_interrupt_handler();
void network_send_packet(const char*, const unsigned int);
void network_format_packet(Packet*, const char*, const unsigned int);
unsigned short network_btoles(unsigned short val);
void network_display_packet(unsigned char*);

typedef unsigned short u16;
typedef unsigned long u32;

unsigned short network_calc_ip_checksum(unsigned char[], const int);
u16 network_calc_udp_checksum (u16, unsigned char*, unsigned char*, char padding, unsigned char*);
void network_send_file(const char* filename);

void network_set_source_port(unsigned short);
void network_set_dest_port(unsigned short);
void network_set_source_ip(unsigned char, unsigned char, unsigned char, unsigned char);
void network_set_dest_ip(unsigned char, unsigned char, unsigned char, unsigned char);

#endif /*NETWORK_H_*/
