#include <stdio.h>
#include <stdlib.h>

typedef unsigned short u16;
typedef unsigned long u32;

u16 udp_sum_calc (u16, u16[], u16[], char padding, u16[]);
u16 rev_endian(u16 val) {
	return ((val & 0x00FF) << 8) | ((val & 0xFF00) >> 8);
}

int
main ()
{

  u16 checksum_fields_be[] = {
    // ip source address
		0x000A, 0x0200,
    // ip destination address
		0x000A, 0x0D00,
    // protocol
		0x1100, 		
    // udp length or total length?
		//0x4600,   // total length
	 0x3200, // udp length
		// source port
		0x3500,
		// dest port
		0x6CB4,
		// length
		0x3200,
		// checksum
		//0x9C79,
		0x000,
		// data
		0xA598, 0x8081,
		0x0100, 0x0100, 0x0000, 0x0000, 0x7804, 0x6B63, 0x0364, 0x6F63,
		0x006D, 0x0100, 0x0100, 0x0CC0, 0x0100, 0x0100, 0x0000, 0x2C33,
		0x0400, 0x1A48, 0x63CB
  };

	int len = (int)(sizeof(checksum_fields_be) / sizeof(u16));
/*
	printf("sizeof(checksum_fields_be) == %d, len = %d\n", 
		(int) sizeof(checksum_fields_be), len);

	for(int i=0; i<len; ++i) {
		printf("%04d: 0x%04X == 0x%04X\n", i,
			(unsigned short) rev_endian(checksum_fields_be[i]),
			(unsigned short) checksum_fields_be[i]
		);
	}
*/
	

  int checksum_be = 0;
  int checksum_le = 0;

  int i;
  for (i = 0; i < len; ++i) {
    checksum_be += checksum_fields_be[i];
    checksum_le += rev_endian(checksum_fields_be[i]);
	}

  checksum_be = ~checksum_be & 0xFFFF;
  checksum_le = ~checksum_le & 0xFFFF;
	
	//printf ("expected = 0x4929\n");
  printf ("checksum_be = 0x%04X\n", checksum_be);
  printf ("checksum_le = 0x%04X\n", checksum_le);

/*

	u16 source[] = { 0x0A00, 0x000d }; // 10.0.0.13
	//u16 source[] = { 0x000A00, 0x0d00 }; // 10.0.0.13
	u16 dest[] = { 0x0A00, 0x000d }; // 10.0.0.13
	//u16 dest[] = { 0x000A, 0x0d00 }; // 10.0.0.13
	char padding = 0;
	u16 length = 12;
	//u16 data[] = { 0xA08B, 0x26CA, 0x000C, 0x1437, 0x5414, 0x0000 };
	u16 data[] = { 0xA08B, 0x26CA, 0x000C, 0x0000, 0x5414, 0x0000 };

	//u16 data[] = { 0x8BA0, 0xCA26, 0x0C00, 0x1437, 0x1454, 0x0000 };
	//u16 data[] = { 0x8BA0, 0xCA26, 0x0C00, 0x0000, 0x1454, 0x0000 };
	u16 checksum = udp_sum_calc(length, source, dest, padding, data);
	printf ("expected = 0x1347\n");
  printf ("checksum = 0x%04X\n", checksum);

	*/
  return 0;
}

u16
udp_sum_calc (u16 len_udp, u16 src_addr[], u16 dest_addr[], char padding,
	      u16 buff[])
{
  u16 prot_udp = 17;
  u16 padd = 0;
  u16 word16;
  u32 sum;
	int i;

  // Find out if the length of data is even or odd number. If odd,
  // add a padding byte = 0 at the end of packet
  if (padding & 1 == 1)
    {
      padd = 1;
      buff[len_udp] = 0;
    }

  //initialize sum to zero
  sum = 0;

  // make 16 bit words out of every two adjacent 8 bit words and 
  // calculate the sum of all 16 vit words
  for (i = 0; i < len_udp + padd; i = i + 2)
    {
      word16 = ((buff[i] << 8) & 0xFF00) + (buff[i + 1] & 0xFF);
      sum = sum + (unsigned long) word16;
    }
  // add the UDP pseudo header which contains the IP source and destinationn addresses
  for (i = 0; i < 4; i = i + 2)
    {
      word16 = ((src_addr[i] << 8) & 0xFF00) + (src_addr[i + 1] & 0xFF);
      sum = sum + word16;
    }
  for (i = 0; i < 4; i = i + 2)
    {
      word16 = ((dest_addr[i] << 8) & 0xFF00) + (dest_addr[i + 1] & 0xFF);
      sum = sum + word16;
    }
  // the protocol number and the length of the UDP packet
  sum = sum + prot_udp + len_udp;

  // keep only the last 16 bits of the 32 bit calculated sum and add the carries
  while (sum >> 16)
    sum = (sum & 0xFFFF) + (sum >> 16);

  // Take the one's complement of sum
  sum = ~sum;

  return ((u16) sum);
}
